#!/usr/bin/php -q
<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2006 Larry Adams                                          |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
*/

/* tick use required as of PHP 4.3.0 to accomodate signal handling */
declare(ticks = 1);

/*	display_help - displays the usage of the function */
function display_help () {
	print "Boost RRD Updater Process Control Version 1.6, Copyright 2006 - Larry Adams\n\n";
	print "usage: poller_boost.php [-d] [-h] [--help] [-v] [-V] [--version]\n\n";
	print "-f               - Force the execution of a update process\n";
	print "-d               - Display verbose output during execution\n";
	print "-v -V --version  - Display this help message\n";
	print "-h --help        - display this help message\n";
}

function sig_handler($signo) {
	switch ($signo) {
		case SIGTERM:
		case SIGINT:
		case SIGHUP:
			cacti_log("WARNING: Boost Poller terminated by user", FALSE, "BOOST");

			/* tell the main poller that we are done */
			db_execute("REPLACE INTO settings (name, value) VALUES ('boost_poller_status', 'terminated - end time:" . date("Y-m-d G:i:s") ."')");

			exit;
			break;
		default:
			/* ignore all other signals */
	}

}

function output_rrd_data($start_time, $force = FALSE) {
	global $start, $max_run_duration, $config, $debug;

	include_once($config["base_path"] . "/lib/rrd.php");

	$boost_poller_status = read_config_option("boost_poller_status");

	$rrds_updates = 0;

	/* detect for a terminated process */
	if (substr_count($boost_poller_status, "running")) {
		$status_array = explode(":", $boost_poller_status);

		if (!empty($status_array[1])) {
			$previous_start_time = strtotime($status_array[1]);

			/* if the runtime was exceeded, allow the next process to run */
			if ($previous_start_time + $max_run_duration < $start_time) {
				cacti_log("ERROR: Detected Poller Boost Abend, Contact support", false, "BOOST SVR");
				$boost_poller_status = "abend";
			}
		}
	}

	/* if the poller is not running, or has never run, start */
	if ((substr_count($boost_poller_status, "timeout")) ||
		(substr_count($boost_poller_status, "complete")) ||
		(substr_count($boost_poller_status, "terminated")) ||
		(substr_count($boost_poller_status, "abend")) ||
		($force) ||
		(empty($boost_poller_status))) {

		/* mark the boost server as running */
		db_execute("REPLACE INTO settings (name, value) VALUES ('boost_poller_status', 'running - start time:" . date("Y-m-d G:i:s") ."')");

		$current_time = date("Y-m-d G:i:s", $start_time);

		$rrdtool_pipe = rrd_init();

		while (1) {
			$rows = db_fetch_cell("SELECT count(*) FROM poller_output_boost WHERE time<='$current_time'");

			if ($rows > 0) {
				$rrds_updates += boost_process_poller_output($rrdtool_pipe, FALSE, "", $current_time);
			}else{
				break;
			}

			if ((time()-$start) > $max_run_duration) {
				cacti_log("FATAL: RRD On Demand Updater Exceeded Runtime Limits!!!");
				break;
			}
		}

		/* tell the main poller that we are done */
		db_execute("REPLACE INTO settings (name, value) VALUES ('boost_poller_status', 'complete - end time:" . date("Y-m-d G:i:s") ."')");

		rrd_close($rrdtool_pipe);
	}

	return $rrds_updates;
}

function log_boost_statistics($rrd_updates) {
	global $start;

	/* take time and log performance data */
	list($micro,$seconds) = split(" ", microtime());
	$end = $seconds + $micro;

	$cacti_stats = sprintf(
		"Time:%01.4f " .
		"RRDUpdates:%s",
		round($end-$start,4),
		$rrd_updates);

	/* log to the database */
	 db_execute("REPLACE INTO settings (name,value) VALUES ('stats_boost', '" . $cacti_stats . "')");

	/* log to the logfile */
	cacti_log("BOOST STATS: " . $cacti_stats , true, "SYSTEM");
}

/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0]) || isset($_SERVER['REQUEST_METHOD'])  || isset($_SERVER['REMOTE_ADDR'])) {
	die("<br><strong>This script is only meant to run at the command line.</strong>");
}

/* We are not talking to the browser */
$no_http_headers = true;

$dir = dirname(__FILE__);
chdir($dir);

if (strpos($dir, 'boost') !== false) {
	chdir('../../');
}

/* include important functions */
if (file_exists("./include/global.php")) {
	include_once("./include/global.php");
} else {
	include_once("./include/config.php");
}
include_once($config["base_path"] . "/lib/poller.php");

/* get the boost polling cycle */
$max_run_duration = read_config_option("boost_rrd_update_max_runtime");

/* process calling arguments */
$parms = $_SERVER["argv"];
array_shift($parms);

$debug          = FALSE;
$forcerun       = FALSE;
$forcerun_maint = FALSE;

foreach($parms as $parameter) {
	@list($arg, $value) = @explode("=", $parameter);

	switch ($arg) {
	case "-d":
		$debug = TRUE;
		break;
	case "-h":
		display_help();
		exit;
	case "-f":
		$forcerun = TRUE;
		break;
	case "-v":
	case "-V":
		display_help();
		exit;
	case "--version":
		display_help();
		exit;
	case "--help":
		display_help();
		exit;
	default:
		print "ERROR: Invalid Parameter " . $parameter . "\n\n";
		display_help();
		exit;
	}
}

/* install signal handlers for UNIX only */
if (function_exists("pcntl_signal")) {
	pcntl_signal(SIGTERM, "sig_handler");
	pcntl_signal(SIGINT, "sig_handler");
	pcntl_signal(SIGHUP, "sig_handler");
}

/* take time and log performance data */
list($micro,$seconds) = split(" ", microtime());
$start = $seconds + $micro;

/* let's give this script lot's of memory, and let it run for ever */
ini_set("max_execution_time", "0");
ini_set("memory_limit", "2048M");

if ((read_config_option("boost_rrd_update_enable") == "on") || $forcerun) {
	/* turn on the system level updates as that is what dictates "on/off" */
	if ((!$forcerun) &&
		(read_config_option("boost_rrd_update_system_enable") != "on")) {
		db_execute("REPLACE INTO settings (name,value)
			VALUES ('boost_rrd_update_system_enable','on')");
	}

	$seconds_offset = read_config_option("boost_rrd_update_interval") * 60;

	/* find out if it's time to collect device information */
	$base_start_time = strtotime(read_config_option("boost_rrd_update_base_time"));
	$last_run_time = strtotime(read_config_option("boost_last_run_time"));
	$next_run_time = strtotime(read_config_option("boost_next_run_time"));
	$previous_base_start_time = strtotime(read_config_option("boost_prev_base_time"));

	/* see if the user desires a new start time */
	if (!empty($previous_base_start_time)) {
		if ($base_start_time <> $previous_base_start_time) {
			unset($last_run_time);
			db_execute("DELETE FROM settings WHERE name='boost_last_run_time'");
		}
	}

	/* set to detect if the user cleared the time between polling cycles */
	db_execute("REPLACE INTO settings (name, value) VALUES ('boost_prev_base_time', '" . date("Y-m-d G:i:s", $base_start_time) . "')");

	/* determine the next start time */
	$current_time = time();
	if (empty($last_run_time)) {
		if ($current_time > $base_start_time) {
			/* if timer expired within a polling interval, then poll */
			if (($current_time - $seconds_offset) < $base_start_time) {
				$next_run_time = $current_time;
			}else{
				$next_run_time = $base_start_time;
				while ($next_run_time < $current_time) {
					$next_run_time += $seconds_offset;
				}
			}
		}else{
			$next_run_time = $base_start_time;
			while ($next_run_time < $current_time) {
				$next_run_time += $seconds_offset;
			}
		}

		/* since the poller has never run before, let's fake it out */
		$last_run_time = $next_run_time - $seconds_offset;
		db_execute("REPLACE INTO settings (name, value) VALUES ('boost_last_run_time', '" . date("Y-m-d G:i:s", $last_run_time) . "')");
	}else{
		$next_run_time = $last_run_time + $seconds_offset;
	}
	$time_till_next_run = $next_run_time - $current_time;

	/* determine if you must output boost table now */
	$max_records = read_config_option("boost_rrd_update_max_records");
	$current_records = db_fetch_cell("SELECT count(*) FROM poller_output_boost");

	if (($time_till_next_run <= 0) ||
		($forcerun) ||
		($current_records >= $max_records) ||
		($next_run_time <= $current_time)) {
		db_execute("REPLACE INTO settings (name, value) VALUES ('boost_last_run_time', '" . date("Y-m-d G:i:s", $current_time) . "')");

		/* output all the rrd data to the rrd files */
		$rrds_updates = output_rrd_data($current_time, $forcerun);
		log_boost_statistics($rrds_updates);

		$next_run_time = $current_time + $seconds_offset;
	}

	/* store the next run time so that people understand */
	db_execute("REPLACE INTO settings (name, value) VALUES ('boost_next_run_time', '" . date("Y-m-d G:i:s", $next_run_time) . "')");
}else{
	/* turn on the system level updates as that is what dictates "off" */
	if (read_config_option("boost_rrd_update_system_enable") == "on") {
		db_execute("REPLACE INTO settings (name,value)
			VALUES ('boost_rrd_update_system_enable','off')");
	}

	$rows = db_fetch_cell("SELECT count(*) FROM poller_output_boost");

	if ($rows > 0) {
		/* determine the time to clear the table */
		$current_time = time();

		/* output all the rrd data to the rrd files */
		$rrds_updates = output_rrd_data($current_time, $forcerun);

		log_boost_statistics($rrds_updates);
	}
}

/* remove stale png's from the cache.  I consider png's stale afer 1 hour */
if ((read_config_option("boost_png_cache_enable") == "on") || $forcerun) {
	$cache_directory = read_config_option("boost_png_cache_directory");
	$remove_time = time() - 3600;

	$directory_contents = array();

	if ($handle = opendir($cache_directory)) {
		/* This is the correct way to loop over the directory. */
		while (false !== ($file = readdir($handle))) {
			$directory_contents[] = $file;
		}

		closedir($handle);
	}

	/* remove age old files */
	if (sizeof($directory_contents)) {
		/* goto the cache directory */
		chdir($cache_directory);

		/* check and fry as applicable */
		foreach($directory_contents as $file) {
			if (is_writable($file)) {
				$modify_time = filemtime($file);
				if ($modify_time < $remove_time) {
					/* only remove jpeg's and png's */
					if ((substr_count(strtolower($file), ".png")) ||
						(substr_count(strtolower($file), ".jpg"))) {
						unlink($file);
					}
				}
			}
		}
	}
}

?>